#include <iostream>
#include "assert.h"
using namespace std;

class Item;

class List
{
   private:
      Item* first;
      Item* last;

   public:
      Item* getFirst() { return first; }
      Item* getLast() { return last; }

   private:
      void link (Item* before, Item* fresh, Item* after);
   
   public:
       void insertFirst(Item* fresh);
       void insertLast(Item* fresh);

       void insertBefore(Item*  target, Item* fresh);
       void insertAfter(Item* target, Item* fresh);

       void unlink (Item* elem);
       void remove (Item* elem);

       Item* find(string name0);
       Item* get (int inx);
       void print();

       List ();
       ~ List ();
};

class Item
{
   public:
       string name;

   private:
       Item* prev;
       Item* next;

   public:
      Item* getPrev() { return prev; }
      Item* getNext() { return next; }

      Item (string name0);

      friend class List;
};

List::List () :
    first (nullptr),
    last (nullptr)
{
}

List :: ~List()
{
    while (first != nullptr)
    {
        remove (first);
    }
}

Item::Item(string name0) :
    name(name0),
    prev(nullptr),
    next(nullptr)
{
}

void List::link(Item* before, Item* fresh, Item* after)
// a - seznam do ktereho vkladame
// fresh - nove vkladany prvek (musi existovat, nesmi byt v zadnem seznamu)
// before - ukazatel na predchozi prvek ( before == nullptr => fresh bude prvni)
// after - ukazatel na nasledujici prvek ( after == nullptr => fresh bude posledni)
{
    fresh->prev = before;
    fresh->next = after;

    if (before != nullptr)
        before->next = fresh; // nekdo je pred
    else
        first = fresh; // jsme prvni

    if (after != nullptr)
        after->prev = fresh;
    else
        last = fresh;
}

void List::insertFirst(Item * fresh)
{
    link (nullptr, fresh, first);
}

void List::insertLast (Item* fresh)
{
    link (last, fresh, nullptr);
}

void List::insertBefore(Item* target, Item* fresh)
{
    assert(target != nullptr);
    link(target->prev, fresh, target);
}

void List::insertAfter(Item* target, Item* fresh)
{
    assert(target != nullptr);
    link (target, fresh, target->next);
}

void List::unlink (Item* elem)
{
    if (elem != nullptr)
    {
        Item* before = elem->prev;
        Item* after = elem->next;

        if (before != nullptr)
            before->next = after;
        else
            first = after;

        if (after != nullptr)
            after->prev = before;
        else
            last = before;

        elem->prev = nullptr;
        elem->next = nullptr;
    }
}

void List::remove(Item* elem)
{
    if (elem != nullptr)
    {
        unlink(elem);
        delete elem;
    }
}

Item* List::get(int inx) // inx == 0, prvni prvek
{
    Item* p = first;
    while (inx > 0 && p != nullptr) {
        inx--;
        p = p->next;
    }
    return p;
}

Item* List::find(string name0)
{
    Item* p = first;
    while (p != nullptr && p->name != name0)
    {
        p = p->next;
    }
    return p;
}

void List::print()
{
    Item* p = first;
    if (p == nullptr)
    {
        cout << "seznam je prazdny" << endl;
    }
    while (p != nullptr)
    {
        cout << p->name << endl;
        p = p->next;
    }
    cout << endl;
}

List seznam;

int main()
{
    seznam.insertFirst (new Item ("cervena"));
    seznam.insertLast (new Item("zelena"));
    seznam.insertBefore (seznam.getLast (), new Item("oranzova"));

    Item* t = seznam.get(1); // index 1 ... druhy prvek
    if (t != nullptr)
       seznam.insertAfter (t, new Item("modra"));
    
    Item* m = seznam.find ("modra");
    if (m != nullptr)
        seznam.insertBefore (m, new Item("zluta"));

    seznam.print ();
}
